@@ -3,20 +3,25 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 'use strict';
 
-const { validateOptions } = require('../deprecated/api-utils');
+const { validateOptions } = require("../deprecated/api-utils");
+const { data } = require("../self");
 
 function Options(options) {
   if ('string' === typeof options)
     options = { url: options };
 
   return validateOptions(options, {
-    url: { is: ["string"] },
+    url: {
+      is: ["string"],
+      map: (v) => v ? data.url(v) : v
+    },
     inBackground: {
-      map: function(v) !!v,
+      map: Boolean,
       is: ["undefined", "boolean"]
     },
     isPinned: { is: ["undefined", "boolean"] },
     isPrivate: { is: ["undefined", "boolean"] },
+    inNewWindow: { is: ["undefined", "boolean"] },
     onOpen: { is: ["undefined", "function"] },
     onClose: { is: ["undefined", "function"] },
     onReady: { is: ["undefined", "function"] },
