@@ -7,7 +7,9 @@
 module.metadata = {
   'stability': 'experimental',
   'engines': {
-    'IceCat': '*'
+    'IceCat': '*',
+    'SeaMonkey': '*',
+    'Thunderbird': '*'
   }
 };
 
