@@ -45,7 +45,8 @@ const panels = require("./panel");
 const { EventEmitter, EventEmitterTrait } = require("./deprecated/events");
 const { Trait } = require("./deprecated/traits");
 const LightTrait = require('./deprecated/light-traits').Trait;
-const { Loader, Symbiont } = require("./content/content");
+const { Loader } = require("./content/loader");
+const { Symbiont } = require("./deprecated/symbiont");
 const { Cortex } = require('./deprecated/cortex');
 const windowsAPI = require("./windows");
 const { WindowTracker } = require("./deprecated/window-utils");
@@ -321,7 +322,7 @@ const WidgetTrait = LightTrait.compose(EventEmitterTrait, LightTrait({
    *         Window that has been closed
    */
   _onWindowClosed: function _onWindowClosed(window) {
-    for each (let view in this._views) {
+    for (let view of this._views) {
       if (view._isInChromeWindow(window)) {
         view.destroy();
         break;
@@ -335,7 +336,7 @@ const WidgetTrait = LightTrait.compose(EventEmitterTrait, LightTrait({
    *         BrowserWindow reference from "windows" module
    */
   getView: function getView(window) {
-    for each (let view in this._views) {
+    for (let view of this._views) {
       if (view._isInWindow(window)) {
         return view._public;
       }
