@@ -13,6 +13,10 @@ const commandModules = [
   "gcli/commands/calllog",
   "gcli/commands/cmd",
   "gcli/commands/cookie",
+  "gcli/commands/csscoverage",
+  "gcli/commands/folder",
+  "gcli/commands/highlight",
+  "gcli/commands/inject",
   "gcli/commands/jsb",
   "gcli/commands/listen",
   "gcli/commands/media",
