@@ -1,4 +1,4 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- tab-width: 2; indent-tabs-mode: nil; js-indent-level: 2 -*- */
 /* vim: set ts=2 sw=2 sts=2 et: */
 
 /* This Source Code Form is subject to the terms of the Mozilla Public
@@ -50,9 +50,9 @@ const ContentPanning = {
   },
 
   _setupListenersForPanning: function cp_setupListenersForPanning() {
-    var events;
-    try {
-      content.document.createEvent('TouchEvent');
+    let events;
+
+    if (content.TouchEvent) {
       events = ['touchstart', 'touchend', 'touchmove'];
       this.watchedEventsType = 'touch';
 #ifdef MOZ_WIDGET_GONK
@@ -65,7 +65,7 @@ const ContentPanning = {
                            .processType == Ci.nsIXULRuntime.PROCESS_TYPE_DEFAULT;
       this.hybridEvents = isParentProcess;
 #endif
-    } catch(e) {
+    } else {
       // Touch events aren't supported, so fall back on mouse.
       events = ['mousedown', 'mouseup', 'mousemove'];
       this.watchedEventsType = 'mouse';
@@ -84,12 +84,30 @@ const ContentPanning = {
   },
 
   handleEvent: function cp_handleEvent(evt) {
-    // Ignore events targeting a <iframe mozbrowser> since those will be
-    // handle by the BrowserElementPanning.js instance of it.
+    // Ignore events targeting an oop <iframe mozbrowser> since those will be
+    // handle by the BrowserElementPanning.js instance in the child process.
     if (evt.target instanceof Ci.nsIMozBrowserFrame) {
       return;
     }
 
+    // For in-process <iframe mozbrowser> the events are not targetting
+    // directly the container iframe element, but some node of the document.
+    // So, the BrowserElementPanning instance of the system app will receive
+    // the sequence of touch events, as well as the BrowserElementPanning
+    // instance in the targetted app.
+    // As a result, multiple mozbrowser iframes will try to interpret the
+    // sequence of touch events, which may results into multiple clicks.
+    let targetWindow = evt.target.ownerDocument.defaultView;
+    let frameElement = targetWindow.frameElement;
+    while (frameElement) {
+      targetWindow = frameElement.ownerDocument.defaultView;
+      frameElement = targetWindow.frameElement;
+    }
+
+    if (content !== targetWindow) {
+      return;
+    }
+
     if (evt.defaultPrevented || evt.multipleActionsPrevented) {
       // clean up panning state even if touchend/mouseup has been preventDefault.
       if(evt.type === 'touchend' || evt.type === 'mouseup') {
@@ -494,7 +512,7 @@ const ContentPanning = {
   },
 
   _recvDoubleTap: function(data) {
-    let data = data.json;
+    data = data.json;
 
     // We haven't received a metrics update yet; don't do anything.
     if (this._viewport == null) {
