@@ -68,25 +68,38 @@ function testConstructorFatalOption(data, expectedString)
 
 function testConstructorEncodingOption(aData, aExpectedString)
 {
+  function errorMessage(encoding) {
+    return `The given encoding '${String(encoding).trim()}' is not supported.`;
+  }
+
   // valid encoding passed
-  testCharset({encoding: "iso-8859-11", input: aData, expected: aExpectedString,
+  var encoding = "iso-8859-11";
+  testCharset({encoding: encoding, input: aData, expected: aExpectedString,
     msg: "decoder testing constructor valid encoding."});
 
-  // invalid encoding passed
-  testCharset({encoding: "asdfasdf", input: aData, error: "TypeError",
-    msg: "constructor encoding, invalid encoding test."});
-
   // passing spaces for encoding
-  testCharset({encoding: "   ", input: aData, error: "TypeError",
+  encoding = "   ";
+  testCharset({encoding: encoding, input: aData, error: "RangeError",
+    errorMessage: errorMessage(encoding),
     msg: "constructor encoding, spaces encoding test."});
 
-  // passing null for encoding
-  testCharset({encoding: null, input: aData, error: "TypeError",
+  // invalid encoding passed
+  encoding = "asdfasdf";
+  testCharset({encoding: encoding, input: aData, error: "RangeError",
+    errorMessage: errorMessage(encoding),
+    msg: "constructor encoding, invalid encoding test."});
+
+  // null encoding passed
+  encoding = null;
+  testCharset({encoding: encoding, input: aData, error: "RangeError",
+    errorMessage: errorMessage(encoding),
     msg: "constructor encoding, \"null\" encoding test."});
 
   // empty encoding passed
-  testCharset({encoding: "", input: aData, error: "TypeError",
-    msg: "constuctor encoding, empty encoding test."});
+  encoding = "";
+  testCharset({encoding: encoding, input: aData, error: "RangeError",
+    errorMessage: errorMessage(encoding),
+    msg: "constructor encoding, empty encoding test."});
 
   // replacement character test
   aExpectedString = "\ufffd\ufffd\ufffd\ufffd\ufffd\ufffd\ufffd\ufffd"
@@ -170,8 +183,8 @@ function testDecodeStreamOption(data, expectedString)
   ], msg: "decode() stream test utf-8."});
 
   testCharset({encoding: "utf-8", fatal: true, array: [
-    {input: [0xC2], error: "EncodingError"},
-    {input: [0x80], error: "EncodingError"},
+    {input: [0xC2], error: "TypeError"},
+    {input: [0x80], error: "TypeError"},
   ], msg: "decode() stream test utf-8 fatal."});
 }
 
@@ -342,7 +355,6 @@ function testDecoderGetEncoding()
     {encoding: "x-mac-cyrillic", labels: ["x-mac-cyrillic", "x-mac-ukrainian"]},
     {encoding: "gbk", labels: ["chinese", "csgb2312", "csiso58gb231280", "gb2312", "gb_2312", "gb_2312-80", "gbk", "iso-ir-58", "x-gbk"]},
     {encoding: "gb18030", labels: ["gb18030"]},
-    {encoding: "hz-gb-2312", labels: ["hz-gb-2312"]},
     {encoding: "big5", labels: ["big5", "cn-big5", "csbig5", "x-x-big5"]},
     {encoding: "big5-hkscs", labels: ["big5-hkscs"]},
     {encoding: "euc-jp", labels: ["cseucpkdfmtjapanese", "euc-jp", "x-euc-jp"]},
@@ -352,7 +364,7 @@ function testDecoderGetEncoding()
     {encoding: "utf-16le", labels: ["utf-16", "utf-16le"]},
     {encoding: "utf-16be", labels: ["utf-16be"]},
     {encoding: "x-user-defined", labels: ["x-user-defined"]},
-    {error: "TypeError", labels: ["x-windows-949", "\u0130SO-8859-1", "csiso2022kr", "iso-2022-kr", "iso-2022-cn", "iso-2022-cn-ext", "replacement"]},
+    {error: "RangeError", labels: ["x-windows-949", "\u0130SO-8859-1", "csiso2022kr", "iso-2022-kr", "iso-2022-cn", "iso-2022-cn-ext", "replacement", "hz-gb-2312"]},
   ];
 
   for (var le of labelEncodings) {
@@ -377,6 +389,9 @@ function testCharset(test)
     var decoder = new TextDecoder(test.encoding, fatal);
   } catch (e) {
     assert_equals(e.name, test.error, test.msg + " error thrown from the constructor.");
+    if (test.errorMessage) {
+      assert_equals(e.message, test.errorMessage, test.msg + " error thrown from the constructor.");
+    }
     return;
   }
 
@@ -440,7 +455,7 @@ function testInvalid2022JP()
       // decode() should never throw unless {fatal: true} is specified
       (new TextDecoder("iso-2022-jp")).decode(new Uint8Array(input));
     } catch (e) {
-      if (e.name !== "EncodingError") {
+      if (e.name !== "TypeError") {
         throw e;
       }
       failureCount++;
