@@ -1,11 +1,11 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 475645;
-var summary = 'Do not crash @ nanojit::LIns::isop';
+var BUGNUMBER = 416737;
+var summary = 'Do not assert: *pc == JSOP_GETARG';
 var actual = '';
 var expect = '';
 
@@ -20,35 +20,12 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-  jit(true);
-
-  if (typeof window != 'undefined')
+  var f = function([]){ function n(){} };
+  if (typeof dis == 'function')
   {
-    var q = (function () { });
-    window.addEventListener("load", q, false);
-    window.onerror = q;
-    arr = new Array();
-    pic = r = new Array;
-    h = t = 7;
-    var pics = "";
-    pic[2] = "";
-    for (i=1; i < pic.length; i++) 
-    {
-      try
-      {
-        if(pics=="")
-          pics=pic[i];
-        else
-          (pic[i]-1 & pic[i].i("") == 1);
-      }
-      catch(ex)
-      {
-      }
-      arr[i]='';
-    }
+    dis(f);
   }
-
-  jit(false);
+  print(f);
 
   reportCompare(expect, actual, summary);
 
