@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -20,10 +20,23 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-// ------- Comment #98 From Gary Kwong [:nth10sd]
+// ------- Comment #138 From Gary Kwong [:nth10sd]
 
-  uneval(function(){(Number(0) for each (NaN in []) for each (x4 in this))});
-// Assertion failure: pos == 0, at ../jsopcode.cpp:2963
+// Does not require -j:
+// ===
+  ((function x(){ yield (x = undefined) } ) for (y in /x/));
+
+// Assertion failure: lexdep->frameLevel() <= funbox->level, at ../jsparse.cpp:1820
+// ===
+  try
+  {
+    for(let x in ( x for (y in x) for each (x in []) )) y;
+  }
+  catch(ex)
+  {
+  }
+// Assertion failure: cg->upvars.lookup(atom), at ../jsemit.cpp:2034
+// ===
 
   reportCompare(expect, actual, summary);
 
