@@ -4,12 +4,13 @@
 /// "Use Terms").   Any redistribution of this code must retain the above
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch10/10.4/10.4.3/10.4.3-1-76gs.js
- * @description Strict - checking 'this' from a global scope (strict function declaration called by Function.prototype.bind()())
+ * @path ch10/10.4/10.4.3/10.4.3-1-79gs.js
+ * @description Strict - checking 'this' from a global scope (strict function declaration called by Function.prototype.bind(someObject)())
  * @onlyStrict
  */
 
-function f() { "use strict"; return this===undefined;};
-if (! (f.bind()())){
+var o = {};
+function f() { "use strict"; return this===o;};
+if (! (f.bind(o)())){
     throw "'this' had incorrect value!";
 }
\ No newline at end of file
