@@ -4,13 +4,13 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch10/10.4/10.4.3/10.4.3-1-77-s.js
- * @description Strict Mode - checking 'this' (strict function declaration called by Function.prototype.bind(null)())
- * @onlyStrict
+ * @path ch10/10.4/10.4.3/10.4.3-1-84-s.js
+ * @description Strict Mode - checking 'this' (non-strict function declaration called by strict new'ed Function constructor)
+ * @noStrict
  */
     
 function testcase() {
-function f() { "use strict"; return this===null;};
-return f.bind(null)();
+fnGlobalObject().f = function()  { return this!==undefined;};
+return (function () {return new Function("\"use strict\";return f();")();})();
 }
 runTestCase(testcase);
\ No newline at end of file
