@@ -4,23 +4,16 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch11/11.4/11.4.1/11.4.1-4-a-4-s.js
- * @description Strict Mode - TypeError isn't thrown when deleting configurable accessor property
+ * @path ch11/11.13/11.13.2/11.13.2-14-s.js
+ * @description Strict Mode - ReferenceError isn't thrown if the LeftHandSideExpression of a Compound Assignment operator(%=) evaluates to a resolvable reference
  * @onlyStrict
  */
 
 
 function testcase() {
         "use strict";
-        var obj = {};
-        Object.defineProperty(obj, "prop", {
-            get: function () {
-                return "abc"; 
-            },
-            configurable: true
-        });
-
-        delete obj.prop;
-        return !obj.hasOwnProperty("prop");
+        var _11_13_2_14 = 5
+        _11_13_2_14 %= 2;
+        return _11_13_2_14 === 1;
     }
 runTestCase(testcase);
