@@ -4,30 +4,17 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch12/12.14/12.14-11.js
- * @description catch introduces scope - name lookup finds inner variable
+ * @path ch12/12.1/12.1-1.js
+ * @description 12.1 - block '{ StatementListopt };' is not allowed: try-catch
  */
 
 
 function testcase() {
-  function f(o) {
-
-    function innerf(o) {
-      var x = 42;
-
-      try {
-        throw o;
-      }
-      catch (e) {
-        return x;
-      }
+        try {
+            eval("try{};catch(){}");
+            return false;
+        } catch (e) {
+            return e instanceof SyntaxError;
+        }
     }
-
-    return innerf(o);
-  }
-  
-  if (f({}) === 42) {
-    return true;
-  }
- }
 runTestCase(testcase);
