@@ -4,23 +4,17 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch12/12.2/12.2.1/12.2.1-14-s.js
- * @description arguments - a function expr declaring a var named 'arguments' throws SyntaxError in strict mode
+ * @path ch12/12.2/12.2.1/12.2.1-6-s.js
+ * @description eval - a Function assigning into 'eval' will not throw any error if contained within strict mode and its body does not start with strict mode
  * @onlyStrict
  */
 
 
-
-
 function testcase() {
   'use strict';
-
-  try {
-    eval('(function (){var arguments;});');
-    return false;
-  }
-  catch (e) {
-    return (e instanceof SyntaxError);
-  }
-}
-runTestCase(testcase);
\ No newline at end of file
+  
+    var f = Function('eval = 42;');
+    f();
+    return true;
+ }
+runTestCase(testcase);
