@@ -13,23 +13,27 @@
 
 #include "webrtc/modules/audio_processing/aec/aec_core.h"
 
-enum { kResamplingDelay = 1 };
-enum { kResamplerBufferSize = FRAME_LEN * 4 };
+enum {
+  kResamplingDelay = 1
+};
+enum {
+  kResamplerBufferSize = FRAME_LEN * 4
+};
 
 // Unless otherwise specified, functions return 0 on success and -1 on error
-int WebRtcAec_CreateResampler(void **resampInst);
-int WebRtcAec_InitResampler(void *resampInst, int deviceSampleRateHz);
-int WebRtcAec_FreeResampler(void *resampInst);
+int WebRtcAec_CreateResampler(void** resampInst);
+int WebRtcAec_InitResampler(void* resampInst, int deviceSampleRateHz);
+int WebRtcAec_FreeResampler(void* resampInst);
 
 // Estimates skew from raw measurement.
-int WebRtcAec_GetSkew(void *resampInst, int rawSkew, float *skewEst);
+int WebRtcAec_GetSkew(void* resampInst, int rawSkew, float* skewEst);
 
 // Resamples input using linear interpolation.
-void WebRtcAec_ResampleLinear(void *resampInst,
-                              const short *inspeech,
+void WebRtcAec_ResampleLinear(void* resampInst,
+                              const float* inspeech,
                               int size,
                               float skew,
-                              short *outspeech,
-                              int *size_out);
+                              float* outspeech,
+                              int* size_out);
 
 #endif  // WEBRTC_MODULES_AUDIO_PROCESSING_AEC_AEC_RESAMPLER_H_
