@@ -8,14 +8,22 @@ package org.mozilla.gecko;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashMap;
+import java.util.Map;
+import java.util.TreeSet;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
 import org.json.JSONException;
 import org.json.JSONObject;
-import org.mozilla.gecko.db.BrowserContract.Bookmarks;
 import org.mozilla.gecko.db.BrowserDB;
+import org.mozilla.gecko.db.URLMetadata;
+import org.mozilla.gecko.favicons.Favicons;
+import org.mozilla.gecko.favicons.LoadFaviconTask;
+import org.mozilla.gecko.favicons.OnFaviconLoadedListener;
+import org.mozilla.gecko.favicons.RemoteFavicon;
+import org.mozilla.gecko.gfx.BitmapUtils;
 import org.mozilla.gecko.gfx.Layer;
+import org.mozilla.gecko.toolbar.BrowserToolbar.TabEditingState;
 import org.mozilla.gecko.util.ThreadUtils;
 
 import android.content.ContentResolver;
@@ -27,50 +35,58 @@ import android.os.Build;
 import android.text.TextUtils;
 import android.util.Log;
 import android.view.View;
+import android.widget.Toast;
 
 public class Tab {
     private static final String LOGTAG = "GeckoTab";
 
     private static Pattern sColorPattern;
     private final int mId;
+    private final BrowserDB mDB;
     private long mLastUsed;
     private String mUrl;
     private String mBaseDomain;
-    private String mUserSearch;
+    private String mUserRequested; // The original url requested. May be typed by the user or sent by an extneral app for example.
     private String mTitle;
     private Bitmap mFavicon;
     private String mFaviconUrl;
-    private int mFaviconSize;
+
+    // The set of all available Favicons for this tab, sorted by attractiveness.
+    final TreeSet<RemoteFavicon> mAvailableFavicons = new TreeSet<>();
     private boolean mHasFeeds;
     private boolean mHasOpenSearch;
-    private SiteIdentity mSiteIdentity;
-    private boolean mReaderEnabled;
+    private final SiteIdentity mSiteIdentity;
     private BitmapDrawable mThumbnail;
-    private int mHistoryIndex;
-    private int mHistorySize;
-    private int mParentId;
-    private boolean mExternal;
+    private final int mParentId;
+    private final boolean mExternal;
     private boolean mBookmark;
-    private boolean mReadingListItem;
+    private boolean mIsInReadingList;
     private int mFaviconLoadId;
     private String mContentType;
     private boolean mHasTouchListeners;
     private ZoomConstraints mZoomConstraints;
     private boolean mIsRTL;
-    private ArrayList<View> mPluginViews;
-    private HashMap<Object, Layer> mPluginLayers;
+    private final ArrayList<View> mPluginViews;
+    private final HashMap<Object, Layer> mPluginLayers;
     private int mBackgroundColor;
     private int mState;
     private Bitmap mThumbnailBitmap;
     private boolean mDesktopMode;
     private boolean mEnteringReaderMode;
-    private Context mAppContext;
+    private final Context mAppContext;
     private ErrorType mErrorType = ErrorType.NONE;
-    private static final int MAX_HISTORY_LIST_SIZE = 50;
     private volatile int mLoadProgress;
-    private volatile int mRecordingCount = 0;
+    private volatile int mRecordingCount;
     private String mMostRecentHomePanel;
 
+    private int mHistoryIndex;
+    private int mHistorySize;
+    private boolean mCanDoBack;
+    private boolean mCanDoForward;
+
+    private boolean mIsEditing;
+    private final TabEditingState mEditingState = new TabEditingState();
+
     public static final int STATE_DELAYED = 0;
     public static final int STATE_LOADING = 1;
     public static final int STATE_SUCCESS = 2;
@@ -93,28 +109,16 @@ public class Tab {
 
     public Tab(Context context, int id, String url, boolean external, int parentId, String title) {
         mAppContext = context.getApplicationContext();
+        mDB = GeckoProfile.get(context).getDB();
         mId = id;
-        mLastUsed = 0;
         mUrl = url;
         mBaseDomain = "";
-        mUserSearch = "";
+        mUserRequested = "";
         mExternal = external;
         mParentId = parentId;
         mTitle = title == null ? "" : title;
-        mFavicon = null;
-        mFaviconUrl = null;
-        mFaviconSize = 0;
-        mHasFeeds = false;
-        mHasOpenSearch = false;
         mSiteIdentity = new SiteIdentity();
-        mReaderEnabled = false;
-        mEnteringReaderMode = false;
-        mThumbnail = null;
         mHistoryIndex = -1;
-        mHistorySize = 0;
-        mBookmark = false;
-        mReadingListItem = false;
-        mFaviconLoadId = 0;
         mContentType = "";
         mZoomConstraints = new ZoomConstraints(false);
         mPluginViews = new ArrayList<View>();
@@ -128,6 +132,7 @@ public class Tab {
         mBackgroundColor = DEFAULT_BACKGROUND_COLOR;
 
         updateBookmark();
+        updateReadingList();
     }
 
     private ContentResolver getContentResolver() {
@@ -159,9 +164,9 @@ public class Tab {
         return mUrl;
     }
 
-    // mUserSearch should never be null, but it may be an empty string
-    public synchronized String getUserSearch() {
-        return mUserSearch;
+    // mUserRequested should never be null, but it may be an empty string
+    public synchronized String getUserRequested() {
+        return mUserRequested;
     }
 
     // mTitle should never be null, but it may be an empty string
@@ -177,6 +182,10 @@ public class Tab {
         return mUrl;
     }
 
+    /**
+     * Returns the base domain of the loaded uri. Note that if the page is
+     * a Reader mode uri, the base domain returned is that of the original uri.
+     */
     public String getBaseDomain() {
         return mBaseDomain;
     }
@@ -219,15 +228,20 @@ public class Tab {
         return mThumbnailBitmap;
     }
 
-    public void updateThumbnail(final Bitmap b) {
+    public void updateThumbnail(final Bitmap b, final ThumbnailHelper.CachePolicy cachePolicy) {
         ThreadUtils.postToBackgroundThread(new Runnable() {
             @Override
             public void run() {
                 if (b != null) {
                     try {
                         mThumbnail = new BitmapDrawable(mAppContext.getResources(), b);
-                        if (mState == Tab.STATE_SUCCESS)
-                            saveThumbnailToDB();
+                        if (mState == Tab.STATE_SUCCESS && cachePolicy == ThumbnailHelper.CachePolicy.STORE) {
+                            saveThumbnailToDB(mDB);
+                        } else {
+                            // If the page failed to load, or requested that we not cache info about it, clear any previous
+                            // thumbnails we've stored.
+                            clearThumbnailFromDB(mDB);
+                        }
                     } catch (OutOfMemoryError oom) {
                         Log.w(LOGTAG, "Unable to create/scale bitmap.", oom);
                         mThumbnail = null;
@@ -257,16 +271,12 @@ public class Tab {
         return mSiteIdentity;
     }
 
-    public boolean getReaderEnabled() {
-        return mReaderEnabled;
-    }
-
     public boolean isBookmark() {
         return mBookmark;
     }
 
-    public boolean isReadingListItem() {
-        return mReadingListItem;
+    public boolean isInReadingList() {
+        return mIsInReadingList;
     }
 
     public boolean isExternal() {
@@ -279,8 +289,8 @@ public class Tab {
         }
     }
 
-    private synchronized void updateUserSearch(String userSearch) {
-        mUserSearch = userSearch;
+    public synchronized void updateUserRequested(String userRequested) {
+        mUserRequested = userRequested;
     }
 
     public void setErrorType(String type) {
@@ -298,6 +308,23 @@ public class Tab {
         mErrorType = type;
     }
 
+    public void setMetadata(JSONObject metadata) {
+        if (metadata == null) {
+            return;
+        }
+
+        final ContentResolver cr = mAppContext.getContentResolver();
+        final URLMetadata urlMetadata = mDB.getURLMetadata();
+
+        final Map<String, Object> data = urlMetadata.fromJSON(metadata);
+        ThreadUtils.postToBackgroundThread(new Runnable() {
+            @Override
+            public void run() {
+                urlMetadata.save(cr, mUrl, data);
+            }
+        });
+    }
+
     public ErrorType getErrorType() {
         return mErrorType;
     }
@@ -310,6 +337,14 @@ public class Tab {
         return mContentType;
     }
 
+    public int getHistoryIndex() {
+        return mHistoryIndex;
+    }
+
+    public int getHistorySize() {
+        return mHistorySize;
+    }
+
     public synchronized void updateTitle(String title) {
         // Keep the title unchanged while entering reader mode.
         if (mEnteringReaderMode) {
@@ -361,46 +396,81 @@ public class Tab {
         return mHasTouchListeners;
     }
 
-    public void setFaviconLoadId(int faviconLoadId) {
-        mFaviconLoadId = faviconLoadId;
-    }
+    public synchronized void addFavicon(String faviconURL, int faviconSize, String mimeType) {
+        RemoteFavicon favicon = new RemoteFavicon(faviconURL, faviconSize, mimeType);
 
-    public int getFaviconLoadId() {
-        return mFaviconLoadId;
-    }
-
-    /**
-     * Returns true if the favicon changed.
-     */
-    public boolean updateFavicon(Bitmap favicon) {
-        if (mFavicon == favicon) {
-            return false;
+        // Add this Favicon to the set of available Favicons.
+        synchronized (mAvailableFavicons) {
+            mAvailableFavicons.add(favicon);
         }
-        mFavicon = favicon;
-        return true;
     }
 
-    public synchronized void updateFaviconURL(String faviconUrl, int size) {
-        // If we already have an "any" sized icon, don't update the icon.
-        if (mFaviconSize == -1)
-            return;
+    public void loadFavicon() {
+        // If we have a Favicon explicitly set, load it.
+        if (!mAvailableFavicons.isEmpty()) {
+            RemoteFavicon newFavicon = mAvailableFavicons.first();
 
-        // Only update the favicon if it's bigger than the current favicon.
-        // We use -1 to represent icons with sizes="any".
-        if (size == -1 || size >= mFaviconSize) {
-            mFaviconUrl = faviconUrl;
-            mFaviconSize = size;
+            // If the new Favicon is different, cancel the old load. Else, abort.
+            if (newFavicon.faviconUrl.equals(mFaviconUrl)) {
+                return;
+            }
+
+            Favicons.cancelFaviconLoad(mFaviconLoadId);
+            mFaviconUrl = newFavicon.faviconUrl;
+        } else {
+            // Otherwise, fallback to the default Favicon.
+            mFaviconUrl = null;
         }
+
+        int flags = (isPrivate() || mErrorType != ErrorType.NONE) ? 0 : LoadFaviconTask.FLAG_PERSIST;
+        mFaviconLoadId = Favicons.getSizedFavicon(mAppContext, mUrl, mFaviconUrl, Favicons.browserToolbarFaviconSize, flags,
+                new OnFaviconLoadedListener() {
+                    @Override
+                    public void onFaviconLoaded(String pageUrl, String faviconURL, Bitmap favicon) {
+                        // The tab might be pointing to another URL by the time the
+                        // favicon is finally loaded, in which case we simply ignore it.
+                        if (!pageUrl.equals(mUrl)) {
+                            return;
+                        }
+
+                        // That one failed. Try the next one.
+                        if (favicon == null) {
+                            // If what we just tried to load originated from the set of declared icons..
+                            if (!mAvailableFavicons.isEmpty()) {
+                                // Discard it.
+                                mAvailableFavicons.remove(mAvailableFavicons.first());
+
+                                // Load the next best, if we have one. If not, it'll fall back to the
+                                // default Favicon URL, before giving up.
+                                loadFavicon();
+
+                                return;
+                            }
+
+                            // Total failure: display the default favicon.
+                            favicon = Favicons.defaultFavicon;
+                        }
+
+                        mFavicon = favicon;
+                        mFaviconLoadId = Favicons.NOT_LOADING;
+                        Tabs.getInstance().notifyListeners(Tab.this, Tabs.TabEvents.FAVICON);
+                    }
+                }
+        );
     }
 
     public synchronized void clearFavicon() {
+        // Cancel any ongoing favicon load (if we never finished downloading the old favicon before
+        // we changed page).
+        Favicons.cancelFaviconLoad(mFaviconLoadId);
+
         // Keep the favicon unchanged while entering reader mode
         if (mEnteringReaderMode)
             return;
 
         mFavicon = null;
         mFaviconUrl = null;
-        mFaviconSize = 0;
+        mAvailableFavicons.clear();
     }
 
     public void setHasFeeds(boolean hasFeeds) {
@@ -415,11 +485,6 @@ public class Tab {
         mSiteIdentity.update(identityData);
     }
 
-    public void setReaderEnabled(boolean readerEnabled) {
-        mReaderEnabled = readerEnabled;
-        Tabs.getInstance().notifyListeners(this, Tabs.TabEvents.MENU_UPDATED);
-    }
-
     void updateBookmark() {
         if (getURL() == null) {
             return;
@@ -433,9 +498,26 @@ public class Tab {
                     return;
                 }
 
-                final int flags = BrowserDB.getItemFlags(getContentResolver(), url);
-                mBookmark = (flags & Bookmarks.FLAG_BOOKMARK) > 0;
-                mReadingListItem = (flags & Bookmarks.FLAG_READING) > 0;
+                mBookmark = mDB.isBookmark(getContentResolver(), url);
+                Tabs.getInstance().notifyListeners(Tab.this, Tabs.TabEvents.MENU_UPDATED);
+            }
+        });
+    }
+
+    void updateReadingList() {
+        if (getURL() == null) {
+            return;
+        }
+
+        ThreadUtils.postToBackgroundThread(new Runnable() {
+            @Override
+            public void run() {
+                final String url = getURL();
+                if (url == null) {
+                    return;
+                }
+
+                mIsInReadingList = mDB.getReadingListAccessor().isReadingListItem(getContentResolver(), url);
                 Tabs.getInstance().notifyListeners(Tab.this, Tabs.TabEvents.MENU_UPDATED);
             }
         });
@@ -449,7 +531,8 @@ public class Tab {
                 if (url == null)
                     return;
 
-                BrowserDB.addBookmark(getContentResolver(), mTitle, url);
+                mDB.addBookmark(getContentResolver(), mTitle, url);
+                Tabs.getInstance().notifyListeners(Tab.this, Tabs.TabEvents.BOOKMARK_ADDED);
             }
         });
     }
@@ -462,31 +545,56 @@ public class Tab {
                 if (url == null)
                     return;
 
-                BrowserDB.removeBookmarksWithURL(getContentResolver(), url);
+                mDB.removeBookmarksWithURL(getContentResolver(), url);
+                Tabs.getInstance().notifyListeners(Tab.this, Tabs.TabEvents.BOOKMARK_REMOVED);
             }
         });
     }
 
     public void addToReadingList() {
-        if (!mReaderEnabled)
-            return;
+        ThreadUtils.postToBackgroundThread(new Runnable() {
+            @Override
+            public void run() {
+                String url = getURL();
+                if (url == null) {
+                    return;
+                }
 
-        JSONObject json = new JSONObject();
-        try {
-            json.put("tabID", String.valueOf(getId()));
-        } catch (JSONException e) {
-            Log.e(LOGTAG, "JSON error - failing to add to reading list", e);
-            return;
-        }
+                mDB.getReadingListAccessor().addBasicReadingListItem(getContentResolver(), url, mTitle);
+                ThreadUtils.postToUiThread(new Runnable() {
+                    @Override
+                    public void run() {
+                        Toast.makeText(mAppContext, R.string.reading_list_added, Toast.LENGTH_SHORT).show();
+                    }
+                });
+            }
+        });
+    }
 
-        GeckoEvent e = GeckoEvent.createBroadcastEvent("Reader:Add", json.toString());
-        GeckoAppShell.sendEventToGecko(e);
+    public void removeFromReadingList() {
+        ThreadUtils.postToBackgroundThread(new Runnable() {
+            @Override
+            public void run() {
+                String url = getURL();
+                if (url == null) {
+                    return;
+                }
+
+                mDB.getReadingListAccessor().removeReadingListItemWithURL(getContentResolver(), url);
+                ThreadUtils.postToUiThread(new Runnable() {
+                    @Override
+                    public void run() {
+                        Toast.makeText(mAppContext, R.string.reading_list_removed, Toast.LENGTH_SHORT).show();
+                    }
+                });
+            }
+        });
     }
 
     public void toggleReaderMode() {
         if (AboutPages.isAboutReader(mUrl)) {
             Tabs.getInstance().loadUrl(ReaderModeUtils.getUrlFromAboutReader(mUrl));
-        } else if (mReaderEnabled) {
+        } else {
             mEnteringReaderMode = true;
             Tabs.getInstance().loadUrl(ReaderModeUtils.getAboutReaderForUrl(mUrl, mId));
         }
@@ -503,7 +611,7 @@ public class Tab {
 
     // Our version of nsSHistory::GetCanGoBack
     public boolean canDoBack() {
-        return mHistoryIndex > 0;
+        return mCanDoBack;
     }
 
     public boolean doBack() {
@@ -515,53 +623,6 @@ public class Tab {
         return true;
     }
 
-    public boolean showBackHistory() {
-        if (!canDoBack())
-            return false;
-        return this.showHistory(Math.max(mHistoryIndex - MAX_HISTORY_LIST_SIZE, 0), mHistoryIndex, mHistoryIndex);
-    }
-
-    public boolean showForwardHistory() {
-        if (!canDoForward())
-            return false;
-        return this.showHistory(mHistoryIndex, Math.min(mHistorySize - 1, mHistoryIndex + MAX_HISTORY_LIST_SIZE), mHistoryIndex);
-    }
-
-    public boolean showAllHistory() {
-        if (!canDoForward() && !canDoBack())
-            return false;
-
-        int min = mHistoryIndex - MAX_HISTORY_LIST_SIZE / 2;
-        int max = mHistoryIndex + MAX_HISTORY_LIST_SIZE / 2;
-        if (min < 0) {
-            max -= min;
-        }
-        if (max > mHistorySize - 1) {
-            min -= max - (mHistorySize - 1);
-            max = mHistorySize - 1;
-        }
-        min = Math.max(min, 0);
-
-        return this.showHistory(min, max, mHistoryIndex);
-    }
-
-    /**
-     * This method will show the history starting on fromIndex until toIndex of the history.
-     */
-    public boolean showHistory(int fromIndex, int toIndex, int selIndex) {
-        JSONObject json = new JSONObject();
-        try {
-            json.put("fromIndex", fromIndex);
-            json.put("toIndex", toIndex);
-            json.put("selIndex", selIndex);
-        } catch (JSONException e) {
-            Log.e(LOGTAG, "JSON error", e);
-        }
-        GeckoEvent e = GeckoEvent.createBroadcastEvent("Session:ShowHistory", json.toString());
-        GeckoAppShell.sendEventToGecko(e);
-        return true;
-    }
-
     public void doStop() {
         GeckoEvent e = GeckoEvent.createBroadcastEvent("Session:Stop", "");
         GeckoAppShell.sendEventToGecko(e);
@@ -569,7 +630,7 @@ public class Tab {
 
     // Our version of nsSHistory::GetCanGoForward
     public boolean canDoForward() {
-        return mHistoryIndex < mHistorySize - 1;
+        return mCanDoForward;
     }
 
     public boolean doForward() {
@@ -581,57 +642,20 @@ public class Tab {
         return true;
     }
 
-    void handleSessionHistoryMessage(String event, JSONObject message) throws JSONException {
-        if (event.equals("New")) {
-            final String url = message.getString("url");
-            mHistoryIndex++;
-            mHistorySize = mHistoryIndex + 1;
-        } else if (event.equals("Back")) {
-            if (!canDoBack()) {
-                Log.w(LOGTAG, "Received unexpected back notification");
-                return;
-            }
-            mHistoryIndex--;
-        } else if (event.equals("Forward")) {
-            if (!canDoForward()) {
-                Log.w(LOGTAG, "Received unexpected forward notification");
-                return;
-            }
-            mHistoryIndex++;
-        } else if (event.equals("Goto")) {
-            int index = message.getInt("index");
-            if (index < 0 || index >= mHistorySize) {
-                Log.w(LOGTAG, "Received unexpected history-goto notification");
-                return;
-            }
-            mHistoryIndex = index;
-        } else if (event.equals("Purge")) {
-            int numEntries = message.getInt("numEntries");
-            if (numEntries > mHistorySize) {
-                Log.w(LOGTAG, "Received unexpectedly large number of history entries to purge");
-                mHistoryIndex = -1;
-                mHistorySize = 0;
-                return;
-            }
-
-            mHistorySize -= numEntries;
-            mHistoryIndex -= numEntries;
-
-            // If we weren't at the last history entry, mHistoryIndex may have become too small
-            if (mHistoryIndex < -1)
-                mHistoryIndex = -1;
-        }
-    }
-
     void handleLocationChange(JSONObject message) throws JSONException {
         final String uri = message.getString("uri");
         final String oldUrl = getURL();
         final boolean sameDocument = message.getBoolean("sameDocument");
         mEnteringReaderMode = ReaderModeUtils.isEnteringReaderMode(oldUrl, uri);
+        mHistoryIndex = message.getInt("historyIndex");
+        mHistorySize = message.getInt("historySize");
+        mCanDoBack = message.getBoolean("canGoBack");
+        mCanDoForward = message.getBoolean("canGoForward");
 
         if (!TextUtils.equals(oldUrl, uri)) {
             updateURL(uri);
             updateBookmark();
+            updateReadingList();
             if (!sameDocument) {
                 // We can unconditionally clear the favicon and title here: we
                 // already filtered both cases in which this was a (pseudo-)
@@ -650,13 +674,12 @@ public class Tab {
         }
 
         setContentType(message.getString("contentType"));
-        updateUserSearch(message.getString("userSearch"));
+        updateUserRequested(message.getString("userRequested"));
         mBaseDomain = message.optString("baseDomain");
 
         setHasFeeds(false);
         setHasOpenSearch(false);
-        updateIdentityData(null);
-        setReaderEnabled(false);
+        mSiteIdentity.reset();
         setZoomConstraints(new ZoomConstraints(true));
         setHasTouchListeners(false);
         setBackgroundColor(DEFAULT_BACKGROUND_COLOR);
@@ -673,8 +696,7 @@ public class Tab {
     void handleDocumentStart(boolean restoring, String url) {
         setLoadProgress(LOAD_PROGRESS_START);
         setState((!restoring && shouldShowProgress(url)) ? STATE_LOADING : STATE_SUCCESS);
-        updateIdentityData(null);
-        setReaderEnabled(false);
+        mSiteIdentity.reset();
     }
 
     void handleDocumentStop(boolean success) {
@@ -683,6 +705,7 @@ public class Tab {
         final String oldURL = getURL();
         final Tab tab = this;
         tab.setLoadProgress(LOAD_PROGRESS_STOP);
+
         ThreadUtils.getBackgroundHandler().postDelayed(new Runnable() {
             @Override
             public void run() {
@@ -699,19 +722,54 @@ public class Tab {
         setLoadProgressIfLoading(LOAD_PROGRESS_LOADED);
     }
 
-    protected void saveThumbnailToDB() {
+    protected void saveThumbnailToDB(final BrowserDB db) {
         final BitmapDrawable thumbnail = mThumbnail;
         if (thumbnail == null) {
             return;
         }
 
         try {
-            String url = getURL();
+            final String url = getURL();
             if (url == null) {
                 return;
             }
 
-            BrowserDB.updateThumbnailForUrl(getContentResolver(), url, thumbnail);
+            db.updateThumbnailForUrl(getContentResolver(), url, thumbnail);
+        } catch (Exception e) {
+            // ignore
+        }
+    }
+
+    public void loadThumbnailFromDB(final BrowserDB db) {
+        try {
+            final String url = getURL();
+            if (url == null) {
+                return;
+            }
+
+            byte[] thumbnail = db.getThumbnailForUrl(getContentResolver(), url);
+            if (thumbnail == null) {
+                return;
+            }
+
+            Bitmap bitmap = BitmapUtils.decodeByteArray(thumbnail);
+            mThumbnail = new BitmapDrawable(mAppContext.getResources(), bitmap);
+
+            Tabs.getInstance().notifyListeners(Tab.this, Tabs.TabEvents.THUMBNAIL);
+        } catch (Exception e) {
+            // ignore
+        }
+    }
+
+    private void clearThumbnailFromDB(final BrowserDB db) {
+        try {
+            final String url = getURL();
+            if (url == null) {
+                return;
+            }
+
+            // Passing in a null thumbnail will delete the stored thumbnail for this url
+            db.updateThumbnailForUrl(getContentResolver(), url, null);
         } catch (Exception e) {
             // ignore
         }
@@ -842,4 +900,16 @@ public class Tab {
     public boolean isRecording() {
         return mRecordingCount > 0;
     }
+
+    public boolean isEditing() {
+        return mIsEditing;
+    }
+
+    public void setIsEditing(final boolean isEditing) {
+        this.mIsEditing = isEditing;
+    }
+
+    public TabEditingState getEditingState() {
+        return mEditingState;
+    }
 }
