@@ -1,4 +1,5 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -10,13 +11,13 @@
 //
 nsresult
 nsQueryArrayElementAt::operator()(const nsIID& aIID, void** aResult) const
-  {
-    nsresult status = mArray
-        ? mArray->QueryElementAt(mIndex, aIID, aResult)
-        : NS_ERROR_NULL_POINTER;
+{
+  nsresult status = mArray ? mArray->QueryElementAt(mIndex, aIID, aResult) :
+                             NS_ERROR_NULL_POINTER;
 
-    if (mErrorPtr)
-      *mErrorPtr = status;
-
-    return status;
+  if (mErrorPtr) {
+    *mErrorPtr = status;
   }
+
+  return status;
+}
